function ydif = sg_difn(yin, gradin, ndif, nzyk, dx)
%SG_DIFN  Differentiation von quidistanten Datenstzen nach SAVITZKY-GOLAY
% Polynom vom Grad grad durch ndif Punkte, gleitende Randpunktbehandlung
% Syntax ydif = sg_difn(yin, gradin, ndif, nzyk, dx)
% Eingabe: dx    : Schrittweite der Daten
%          yin   : zu differenzierender Vektor
%          gradin: Grad des Polynoms
%          ndif  : Ableitungsintervall (ungerade)
%          nzyk  : Ordnung der Ableitung
% Ausgabe: ydif  : differenzierter Vektor
% Externa: difc(grad, n): Funktion zur Berechnung der SAVGOL-Koeffizienten
%
% SAVITZKY-GOLAY Differentiation mit Filter und Berechnung der Filterkoeffizienten abgeleitet aus:
% W. Gander, J. HREBICEK: Solving Problems in Scientific Computing using Maple and Matlab,
% 2nd Ed. (Chapter 9), Springer Heidelberg 1995
%
% letzte nderung 10.02.2000	S. Baunack

% Vereinbarungen und Initialisierungen
cdif = (ndif - 1) /2;	% Zahl der Punkte rechts bzw. links
h = zeros(size(yin));	% leere Matrix
ndat = length(yin);
y = yin;
grad= gradin;		% ursprnglicher Grad des Polynoms (wird am Rand verndert)

while nzyk > 0,
	nzyk = nzyk - 1;

	% Ableitung Mitte (ohne Randpunkte)
	y1 = zeros(2*cdif+1,1);
	c = difc(grad, ndif) /dx;

	% Filterung
	h = filter(c(ndif:-1:1), 1, y);
	h(1:cdif) = y(1:cdif);
	h(cdif+1:ndat-cdif) = h(ndif:ndat);
	h(ndat-cdif+1:ndat) = y(ndat-cdif+1:ndat);

	% gleitende Randpunkte untere Grenze
	for i = 2:cdif,
		ndf = 0;
		% test1 <= 0 keine Ableitung
		% test2 <= 0 keine Ableitung
		test1 = i - cdif;
		test2 = i - ndat + cdif + 1;
		if (test1 > 0),
			if (test2 < 0),
				ndf = cdif;
			else
				ndf = ndat - i;
			end
		else
			ndf = i - 1;
		end
		y1 = zeros(2*ndf+1,1);
		y1 = y(i-ndf:i+ndf);
		grad = gradin;
		if grad >= 2*ndf+1,	% Grad des Polynoms reduzieren
			while grad >= 2*ndf+1,
				grad = grad - 1;
			end
		end
		c = difc(grad,2*ndf+1) /dx;
		h(i) = c' * y1;
	end

	% gleitende Randpunkte obere Grenze
	for i = ndat+1-cdif:ndat-1,
		ndf = 0;
		% test1 <= 0 keine Ableitung
		% test2 <= 0 keine Ableitung
		test1 = i - cdif;
		test2 = i - ndat + cdif + 1;
		if (test1 > 0),
			if (test2 < 0),
				ndf = cdif;
			else
				ndf = ndat - i;
			end
		else
			ndf = i - 1;
		end
		y1 = zeros(2*ndf+1,1);
		y1 = y(i-ndf:i+ndf);
		grad = gradin;
		if grad >= 2*ndf+1,	% Grad des Polynoms reduzieren
			while grad >= 2*ndf+1,
				grad = grad - 1;
			end
		end
		c = difc(grad,2*ndf+1) /dx;
		h(i) = c' * y1;
	end

	% ussere Randpunkte: Anstieg fortsetzen
	h(1) = h(2);
	h(ndat) = h(ndat-1);
	% Rckspeichern Ableitung
	y = h;
end
ydif = y;