function ysmo = sg_smon(yin, gradin, nsmo, nzyk)
%SG_SMON Funktion zur Glttung von quidistanten Datenstzen nach SAVITZKY-GOLAY
% Polynom vom Grad grad durch nsmo Punkte, gleitende Randpunktbehandlung
% Syntax ysmo = sg_smon(yin, gradin, nsmo, nzyk)
% Eingabe: yin   : zu glttender Vektor
%          gradin: Grad des Polynoms (wird am Rand reduziert)
%          nsmo  : Glttungsintervall (ungerade)
%          nzyk  : Zahl der Glttungszyklen
% Ausgabe: ysmo  : gegltteter Vektor
% Externa: smoc(grad, n) : Funktion zur Berechnung der SAVGOL-Koeffizienten
%
% SAVITZKY-GOLAY Glttung mit Filter  und Berechnung der Filterkoeffizienten entnommen aus:
% W. Gander, J. HREBICEK: Solving Problems in Scientific Computing using Maple and Matlab,
% 2nd Ed. (Chapter 9), Springer Heidelberg 1995
%
% letzte nderung 10.02.2000	S. Baunack

% Vereinbarungen und Initialisierungen
csmo = (nsmo - 1) /2;
ndat = length(yin);
h = zeros(size(yin));
y = yin;
grad= gradin;		% ursprnglicher Grad des Polynoms (wird am Rand verndert)

while nzyk > 0,
	nzyk = nzyk - 1;

	% Glttung Mitte (ohne Randpunkte)
	y1 = zeros(2*csmo+1,1);
	c = smoc(grad, nsmo);

	% Filterung
	h = filter(c(nsmo:-1:1), 1, y);
	h(1:csmo) = y(1:csmo);
	h(csmo+1:ndat-csmo) = h(nsmo:ndat);
	h(ndat-csmo+1:ndat) = y(ndat-csmo+1:ndat);

	% gleitende Randpunkte untere Grenze
	for i = 2:csmo,
 		nsm = 0;
		% test1 <= 0 keine Glttung
		% test2 <= 0 keine Glttung
		test1 = i - csmo;
		test2 = i - ndat + csmo + 1;
		if (test1 > 0),
			if (test2 < 0),
				nsm = csmo;
			else
				nsm = ndat - i;
			end
		else
			nsm = i - 1;
		end
		y1 = zeros(2*nsm+1,1);
		y1 = y(i-nsm:i+nsm);
		grad = gradin;
		if grad >= 2*nsm+1,	% Grad des Polynoms reduzieren
			while grad >= 2*nsm+1,
				grad = grad - 1;
			end
		end
		c = smoc(grad,2*nsm+1);
		h(i) = c' * y1;
	end

	% gleitende Randpunkte obere Grenze
	for i = ndat+1-csmo:ndat-1,
		nsm = 0;
		% test1 <= 0 keine Glttung
		% test2 <= 0 keine Glttung
		test1 = i - csmo;
		test2 = i - ndat + csmo + 1;
		if (test1 > 0),
			if (test2 < 0),
				nsm = csmo;
			else
				nsm = ndat - i;
			end
		else
			nsm = i - 1;
		end
		y1 = zeros(2*nsm+1,1);
		y1 = y(i-nsm:i+nsm);
		grad = gradin;
		if grad >= 2*nsm+1,	% Grad des Polynoms reduzieren
			while grad >= 2*nsm+1,
				grad = grad - 1;
			end
		end
		c = smoc(grad,2*nsm+1);
		h(i) = c' * y1;
	end

	h(1) = (y(1)+y(2))/2;	% ussere Randpunkte: Mittelwerte
	h(ndat) = (y(ndat-1)+y(ndat))/2;

	y = h;	% Rckspeichern Glttung
end
ysmo = y;
